from pytest import fail
from unittest.mock import MagicMock
from neterr import (
    SocketErrors,
    StrictHTTPErrors,
    AmbiguousHTTPErrors,
    HTTPErrors
)


def test_socket_exception():
    try:
        raise SocketErrors[0]()
    except SocketErrors:
        pass


def test_strict_http_exception():
    try:
        raise StrictHTTPErrors[0]()
    except StrictHTTPErrors:
        pass


def test_ambiguous_http_exception():
    try:
        raise AmbiguousHTTPErrors[0](MagicMock())
    except AmbiguousHTTPErrors:
        pass


def test_http_exception():
    try:
        raise HTTPErrors[0]()
    except HTTPErrors:
        pass


def test_strict_is_http():
    try:
        raise StrictHTTPErrors[0]()
    except HTTPErrors:
        pass

def test_ambiguous_is_http():
    try:
        raise AmbiguousHTTPErrors[0](MagicMock())
    except HTTPErrors:
        pass

def test_ambigious_is_not_strict():
    for error in AmbiguousHTTPErrors:
        try:
            raise error(MagicMock())
        except StrictHTTPErrors:
            fail('Ambiguous cannot be caught by strict')
        except HTTPErrors:
            pass



from setuptools import setup, find_packages

with open('README.rst') as f:
    readme = f.read()

setup(
    name='neterr',
    description='Useful groupings of exceptions for working with network requests.',
    long_description=readme,
    packages=find_packages(exclude=['tests']),
    use_scm_version=True,
    author='Matt Wisniewski',
    author_email='hello@mattw.life',
    url='https://github.com/polishmatt/neterr',
    keywords=[
        'network',
        'api',
        'http',
        'request'
    ],
    license='Apache License 2.0',
    setup_requires=[
        'pytest-runner',
        'setuptools_scm',
    ],
    tests_require=[
        'pytest',
        'pytest-flake8',
        'pytest-cov',
        'requests',
    ],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
)



import socket
import ssl
import http.client
import urllib.error

__all__ = [
    'SocketErrors',
    'StrictHTTPErrors',
    'HTTPErrors',
    'AmbiguousHTTPErrors'
]

SocketErrors = [
    ConnectionError,
    socket.timeout
]

StrictHTTPErrors = SocketErrors + [
    ssl.SSLError,
    http.client.IncompleteRead,
    http.client.BadStatusLine,
    http.client.LineTooLong,
]

AmbiguousHTTPErrors = [
    urllib.error.URLError
]

try:
    import requests.exceptions
    StrictHTTPErrors += [
        requests.exceptions.ConnectionError,
        requests.exceptions.Timeout,
        requests.exceptions.TooManyRedirects,
        requests.exceptions.ChunkedEncodingError
    ]
    AmbiguousHTTPErrors += [
        requests.exceptions.HTTPError
    ]
except ImportError:
    pass

HTTPErrors = StrictHTTPErrors + AmbiguousHTTPErrors

for error in __all__:
    locals()[error] = tuple(locals()[error])



import sys
import requests
from unittest.mock import patch
import importlib
import neterr


def test_with_requests():
    importlib.reload(neterr)
    assert requests.exceptions.ConnectionError in neterr.StrictHTTPErrors


def test_without_requests():
    raw_import = __import__
    def mock_import(name, *args):
        if name.startswith('requests'):
            raise ImportError()
        return raw_import(name, *args)

    with patch('builtins.__import__', side_effect=mock_import):
        importlib.reload(neterr)
    assert requests.exceptions.ConnectionError not in neterr.StrictHTTPErrors
